@@ -46,6 +46,7 @@ Config::initDefaultValues (void)
 {
     m_orientation = IBUS_ORIENTATION_HORIZONTAL;
     m_page_size = 5;
+    m_ctrl_switch = FALSE;
     m_shift_select_candidate = FALSE;
     m_minus_equal_page = TRUE;
     m_comma_period_page = TRUE;
@@ -68,25 +69,6 @@ Config::initDefaultValues (void)
 void
 Config::readDefaultValues (void)
 {
-#if defined(HAVE_IBUS_CONFIG_GET_VALUES)
-    /* read all values together */
-    initDefaultValues ();
-    GVariant *values =
-            ibus_config_get_values (get<IBusConfig> (), m_section.c_str ());
-    g_return_if_fail (values != NULL);
-
-    GVariantIter iter;
-    gchar *name;
-    GVariant *value;
-    g_variant_iter_init (&iter, values);
-    while (g_variant_iter_next (&iter, "{sv}", &name, &value)) {
-        valueChanged (m_section, name, value);
-        g_free (name);
-        g_variant_unref (value);
-    }
-    g_variant_unref (values);
-#else
-#endif
 }
 
 inline bool
