@@ -44,6 +44,7 @@ public:
     guint option (void) const                   { return m_option & m_option_mask; }
     guint orientation (void) const              { return m_orientation; }
     guint pageSize (void) const                 { return m_page_size; }
+    gboolean ctrlSwitch(void) const             { return m_ctrl_switch; }
     gboolean shiftSelectCandidate (void) const  { return m_shift_select_candidate; }
     gboolean minusEqualPage (void) const        { return m_minus_equal_page; }
     gboolean commaPeriodPage (void) const       { return m_comma_period_page; }
@@ -88,6 +89,7 @@ protected:
 
     gint m_orientation;
     guint m_page_size;
+    gboolean m_ctrl_switch;
     gboolean m_shift_select_candidate;
     gboolean m_minus_equal_page;
     gboolean m_comma_period_page;
